"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _padStart = _interopRequireDefault(require("lodash/padStart"));

var _graphql = require("graphql");

var _language = require("graphql/language");

var _graphqlTypeJson = _interopRequireDefault(require("graphql-type-json"));

var _graphqlUpload = require("graphql-upload");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _default = {
  Mutation: {
    ping: () => 'pong'
  },
  Query: {
    hello: () => 'Oooors!'
  },
  DateTime: new _graphql.GraphQLScalarType({
    name: 'DateTime',
    description: 'DateTime custom scalar type',
    parseValue: value => new Date(value),

    parseLiteral(ast) {
      if (ast.kind === _language.Kind.INT) {
        return parseInt(ast.value, 10);
      }

      if (ast.kind === _language.Kind.STRING) {
        return new Date(ast.value);
      }

      return null;
    },

    serialize: value => {
      const date = value instanceof Date ? value : new Date(value);
      return date.toISOString();
    }
  }),
  Date: new _graphql.GraphQLScalarType({
    name: 'Date',
    description: 'Date custom scalar type',
    parseValue: value => new Date(value),

    parseLiteral(ast) {
      if (ast.kind !== _language.Kind.STRING || !ast.value.match(/^\d{4}-\d{2}-\d{2}$/)) {
        throw new Error('Invalid date format');
      }

      const date = new Date(ast.value);

      if (date.toString() === 'Invalid Date') {
        throw new Error('Invalid Date');
      }

      return date;
    },

    serialize: value => {
      const date = value instanceof Date ? value : new Date(value);

      if (date.toString() === 'Invalid Date') {
        throw new Error('Invalid Date');
      }

      return [date.getFullYear(), (0, _padStart.default)(date.getMonth() + 1, 2, 0), (0, _padStart.default)(date.getDate(), 2, 0)].join('-');
    }
  }),
  JSON: _graphqlTypeJson.default,
  Upload: _graphqlUpload.GraphQLUpload
};
exports.default = _default;