"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _invariant = _interopRequireDefault(require("invariant"));

var _set = _interopRequireDefault(require("lodash/set"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class Resolvers {
  constructor() {
    this.resolvers = {};
    this.middlewares = [];
    this.load();
  }

  addMiddleware(middleware, matcher = /.*/) {
    this.middlewares.push({
      matcher: typeof matcher === 'string' ? new RegExp(`^${matcher}$`) : matcher,
      middleware
    });
    return this;
  }

  load() {
    Object.keys(this).forEach(propr => {
      if (typeof this[propr] === 'function' && this[propr].isResolver) {
        const {
          definition
        } = this[propr];
        this.addResolver(_objectSpread({
          field: propr
        }, definition, {
          resolver: this[propr].bind(this)
        }));
      }
    });
  }

  addResolver({
    path,
    resolver,
    type,
    field
  }) {
    (0, _invariant.default)(typeof resolver === 'function', `"resolver" has to be a function (received ${typeof resolver} instead)`);
    const resolverPath = path || `${type}.${field}`;
    (0, _invariant.default)(path.length, `Invalid resolver path. You must provide a path as either a "path" option or a "type" 
      and "field"`);
    this.resolvers[resolverPath] = resolver;
  }

  dump() {
    return Object.keys(this.resolvers).reduce((acc, path) => {
      const middlewares = this.middlewares.filter(({
        matcher
      }) => matcher.test(path));
      const resolver = middlewares.reverse().reduce((middlewaresAcc, {
        middleware
      }) => (...args) => middleware(...args, middlewaresAcc), this.resolvers[path]);
      (0, _set.default)(acc, path, resolver);
      return acc;
    }, {});
  }

}

var _default = Resolvers;
exports.default = _default;