"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.resolver = exports.loadResolvers = void 0;

var _omit = _interopRequireDefault(require("lodash/omit"));

var _path = _interopRequireDefault(require("path"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const getExports = filePath => {
  const exportMap = require(filePath); // eslint-disable-line


  return Object.assign(exportMap.default || {}, (0, _omit.default)(exportMap, 'default'));
};

const loadResolvers = async dirPath => {
  const files = await _fsExtra.default.readdir(dirPath);
  const map = {};
  await Promise.all(files.map(async file => {
    const filePath = _path.default.join(dirPath, file);

    const stat = await _fsExtra.default.stat(filePath);
    const isDirectory = stat && stat.isDirectory();

    if (isDirectory) {
      map[file] = await loadResolvers(filePath);
      return;
    }

    const extension = _path.default.extname(file);

    if (extension !== '.js') {
      return;
    }

    const filename = file.substr(0, file.length - extension.length);
    const exportMap = getExports(filePath);

    if (filename === 'index') {
      Object.assign(map, exportMap);
    } else {
      map[filename] = exportMap;
    }
  }));
  return map;
};

exports.loadResolvers = loadResolvers;

const resolver = definition => (target, propr, descriptor) => {
  Object.assign(descriptor, {
    enumerable: true
  });
  Object.assign(descriptor.value, {
    isResolver: true,
    definition
  });
  return descriptor;
};

exports.resolver = resolver;